namespace BPA.MES.Base.Application
{
    public interface ISysCacheService
    {
        Task AddCacheKey(string cacheKey);
        Task DelCacheKey(string cacheKey);
        bool Exists(string cacheKey);
        Task<List<string>> GetAllCacheKeys();
        Task<T> GetAsync<T>(string cacheKey);
        Task<string> GetStringAsync(string cacheKey);
        Task RemoveAsync(string key);
        Task SetAsync(string cacheKey, object value);
        Task SetStringAsync(string cacheKey, string value);
    }
}
